/**
 * Machikomake
 * (c) Kenta Cho (cs8k-cyu@asahi-net.or.jp)
 * http://www.asahi-net.or.jp/~cs8k-cyu/
 *
 * displaymanager.c(handle screen buffer)
 */

#include <PalmOS.h>
#include "mckm.h"

WinHandle screenBufferH;
RectangleType sourceBounds;

#define SCREEN_WIDTH 160
#define SCREEN_HEIGHT 160

/* bitmap handlers */
static MemHandle objectBitmapHandles[bitmapTypeCount];
BitmapPtr objectBitmapPtr[bitmapTypeCount];
static WinHandle objectWindowHandles[bitmapTypeCount];

/**
 * create off screen buffer
 */
void createScreenBuffer() {
  UInt16 error;
  WinHandle oldDrawWinH;

  screenBufferH = WinCreateOffscreenWindow
    (SCREEN_WIDTH, SCREEN_HEIGHT, screenFormat, &error);
  sourceBounds.topLeft.x = sourceBounds.topLeft.y = 0;
  sourceBounds.extent.x = SCREEN_WIDTH;
  sourceBounds.extent.y = SCREEN_HEIGHT;
  oldDrawWinH = WinGetDrawWindow();
  WinSetDrawWindow(screenBufferH);
  WinEraseRectangle(&sourceBounds, 0);
  WinSetDrawWindow(oldDrawWinH);
}

void clearDisplay() {
  WinEraseRectangle(&sourceBounds, 0);
}

static WinHandle oldDrawWinH;

void startDrawOffscreen() {
  oldDrawWinH = WinGetDrawWindow();
  WinSetDrawWindow(screenBufferH);
}

void endDrawOffscreen() {
  WinSetDrawWindow(oldDrawWinH);
}

void flipDisplay() {
  WinCopyRectangle
    (screenBufferH, NULL, &sourceBounds, 0, 0, winPaint);
}

/**
 * load bitmaps and hold these handlers
 */
void loadBitmaps() {
  int i;
  WinHandle oldDrawWinH;
  UInt16 error;
  oldDrawWinH = WinGetDrawWindow();
  for ( i=0; i<bitmapTypeCount; i++ ) {
    objectBitmapHandles[i] = DmGetResource(bitmapRsc, FIRST_BITMAP_ID+i);
    objectBitmapPtr[i] = MemHandleLock(objectBitmapHandles[i]);
    objectWindowHandles[i] = WinCreateOffscreenWindow
      (objectBitmapPtr[i]->width, objectBitmapPtr[i]->height,
       screenFormat, &error);
    ErrFatalDisplayIf(error, "Error loading images");
    WinSetDrawWindow(objectWindowHandles[i]);
    WinDrawBitmap(objectBitmapPtr[i], 0, 0);
  }
  WinSetDrawWindow(oldDrawWinH);
}

/**
 * relase bitmap handlers
 */
void unloadBitmaps() {
  int i;
  for ( i=0; i<bitmapTypeCount; i++ ) {
    MemPtrUnlock(objectBitmapPtr[i]);
    DmReleaseResource(objectBitmapHandles[i]);
    if (objectWindowHandles[i]) 
      WinDeleteWindow(objectWindowHandles[i], false);
  }
}


void clearRectangle(int x, int y, int width, int height)  {
  RectangleType rect;
  rect.topLeft.x = x;
  rect.topLeft.y = y;
  rect.extent.x = width;
  rect.extent.y = height;
  WinEraseRectangle(&rect, 0);
}

void drawThickLine(int sx, int sy, int dx, int dy) {
  Boolean hr = FALSE;
  if ( abs(sx-dx) < abs(sy-dy) ) {
    hr = TRUE;
  }
  WinDrawLine(sx, sy, dx, dy);
  if ( hr ) {
    WinDrawLine(sx+1, sy, dx+1, dy);
  } else {
    WinDrawLine(sx, sy+1, dx, dy+1);
  }
}
