/**
 * Machikomake
 * (c) Kenta Cho (cs8k-cyu@asahi-net.or.jp)
 * http://www.asahi-net.or.jp/~cs8k-cyu/
 *
 * gamemanager.c(game manager)
 */

#include <PalmOS.h>
#include "mckm.h"

#define BALL_MAX 4
#define OK_CNT 8

static Ball ball[BALL_MAX];
static int ballNum;
static Vector stick;
static MessageBoard okBoard;
static GameStatus gameStatus;

#define STR_LGT 100

static void initBall(Ball *ball) {
  ball->vel.x = randN(768)-384;
  ball->vel.y = randN(768)-384;
  ball->strLgt = STR_LGT<<RSL;
  ball->d = ball->pd = 0;
}

static void initAllBalls() {
  int i;
  for ( i=0 ; i<ballNum ; i++ ) {  
    initBall(&ball[i]);
  }
}

void startGame() {
  int i;
  gameStatus.status = GAME_IN_PROGRESS;
  gameStatus.score = 0;
  gameStatus.time = gameStatus.endCnt = 0;
  ballNum = 1;
  initAllBalls();
  stick.x = stick.y = 80<<RSL;
  for ( i=0 ; i<BALL_MAX ; i++ ) {
    ball[i].pos.x = stick.x + (randN(STR_LGT>>3)<<RSL);
    ball[i].pos.y = stick.y + (randN(STR_LGT>>3)<<RSL);
  }
  okBoard.cnt = -1;
}

void initGame() {
  FrmGotoForm(GAME_FORM);
}

void startTitle() {
  MemHandle strH;
  char *rscStr;
  char tmStr[10];
  char kyStr[18];
  char nmStr[4];
  int rank;
  int sc, msc;
  if ( gameStatus.time == 0 ) return;
  msc = gameStatus.time*INTERVAL;
  if ( msc < 0 ) msc = 9999;
  rank = (msc-1400)/(1200/12);
  if ( rank < 0 ) rank = 0;
  else if ( rank > 11 ) rank = 11;
  sc = msc/100; msc %= 100;
  if ( sc >= 100 ) {
    sc = msc = 99;
  }
  StrIToA(nmStr, sc);
  StrCopy(tmStr, nmStr);
  StrCat(tmStr, ".");
  StrIToA(nmStr, msc);
  if ( StrLen(nmStr) < 2 ) {
    StrCat(tmStr, "0");
  }
  StrCat(tmStr, nmStr);
  StrCat(tmStr, " ");
  strH = DmGetResource(strRsc, SEC_STR);
  rscStr = MemHandleLock(strH);
  StrCat(tmStr, rscStr);
  MemHandleUnlock(strH);
  DmReleaseResource(strH);
  WinDrawChars(tmStr, StrLen(tmStr), 65, 80);
  strH = DmGetResource(strRsc, DGR1_STR+rank);
  rscStr = MemHandleLock(strH);
  StrCopy(kyStr, rscStr);
  MemHandleUnlock(strH);
  DmReleaseResource(strH);
  strH = DmGetResource(strRsc, DGR_STR);
  rscStr = MemHandleLock(strH);
  StrCat(kyStr, rscStr);
  MemHandleUnlock(strH);
  DmReleaseResource(strH);
  WinDrawChars(kyStr, StrLen(kyStr), 80-StrLen(kyStr)*2, 100);
}  

void initTitle() {
  gameStatus.status = TITLE;
  FrmGotoForm(TITLE_FORM);
}

void initFirst() {
  gameStatus.time = 0;
}

/**
 * draw ok board
 */
static void moveOkBoard() {
  int bmId = OK_CNT-okBoard.cnt;
  if ( bmId > 3 ) bmId = 3;
  okBoard.y--;
  WinDrawBitmap
    (objectBitmapPtr[OK1_BITMAP_IDX+bmId], okBoard.x, okBoard.y);
  okBoard.cnt--;
}  

/**
 * draw shouno-zi
 */
static int shouPos[5][4] = {
  {3,1,18,0}, {11,2,10,18}, {11,10,17,10}, {4,11,5,17}, {0,18,19,18}
};

static void drawShou(int n) {
  int i;
  for ( i=0 ; i<n ; i++ ) {
    if ( i<5 ) {
      drawThickLine(138+shouPos[i][0], 2+shouPos[i][1], 
		    138+shouPos[i][2], 2+shouPos[i][3]);
    } else {
      drawThickLine(138+shouPos[i-5][0], 24+shouPos[i-5][1], 
		    138+shouPos[i-5][2], 24+shouPos[i-5][3]);
    }
  }
}

/*
 * move balls
 */
#define BALL_RADIUS 8
#define GRAV 10
#define AIRREGIST 3
#define SPRING 13

static Vector ps;

static int windCnt;
static int stkIdx;

static void moveBall(Ball *ball) {
  Vector ft, ofs;
  Vector *vel = &(ball->vel);
  long l;
  int nd, ofd;

  ball->tns = FALSE;
  ft.x = 0; ft.y = GRAV;
  ofs.x = ball->pos.x - stick.x;
  ofs.y = ball->pos.y - stick.y;

  if ( gameStatus.status == GAME_IN_PROGRESS ) {
    l = (long)ofs.x*ofs.x+(long)ofs.y*ofs.y;
    if ( l >= (long)ball->strLgt*ball->strLgt ) {
      long cs;
      long spftx, spfty;
      long splgt;
      spftx = ofs.x; spfty = ofs.y;
      splgt = abs(ofs.x)+abs(ofs.y)-ball->strLgt;
      spftx *= splgt; spfty *= splgt;
      spftx >>= SPRING; spfty >>= SPRING;
      ft.x -= spftx; ft.y -= spfty;

      cs = vctInnerProduct(vel, &ofs);
      if ( cs > 0 ) {
	Vector csft = vctGetElement(vel, &ofs);
	vctMul(&csft, 3);
	csft.x >>= 2; csft.y >>= 2;
	vctSub(&ft, &csft);
      }
      cs = vctInnerProduct(&ps, &ofs);
      if ( cs > 0 ) {
	Vector smvft = vctGetElement(&ps, &ofs);
	vctSub(&ft, &smvft);
      }
      ball->tns = TRUE;
    } else if ( l <= (long)(BALL_RADIUS<<RSL)*(BALL_RADIUS<<RSL) ) {
      long cs = vctInnerProduct(vel, &ofs);
      if ( cs < 0 ) {
	Vector csft = vctGetElement(vel, &ofs);
	vctMul(&csft, 3);
	csft.x >>= 1; csft.y >>= 1;
	vctSub(&ft, &csft);
      }
      cs = vctInnerProduct(&ps, &ofs);
      if ( cs > 0 ) {
	Vector smvft = vctGetElement(&ps, &ofs);
	vctSub(&ft, &smvft);
      }
    }
  }
  ft.x -= vel->x>>AIRREGIST;
  ft.y -= vel->y>>AIRREGIST;
  vctAdd(vel, &ft);
  vctAdd(&(ball->pos), vel);

  ofs.x = ball->pos.x - stick.x;
  ofs.y = ball->pos.y - stick.y;
  nd = getDeg(ofs.x, ofs.y);
  if ( ball->tns ) {
    l = (long)ofs.x*ofs.x+(long)ofs.y*ofs.y;
    if ( l >= (long)ball->strLgt*ball->strLgt ) {
      ball->pos.x = stick.x + (((long)sintbl[nd&255]*ball->strLgt)>>8);
      ball->pos.y = stick.y - (((long)costbl[nd&255]*ball->strLgt)>>8);
    }
  } else if ( gameStatus.status == GAME_IN_PROGRESS ) {
    int sl = (ball->strLgt*5)>>2;
    l = (long)ofs.x*ofs.x+(long)ofs.y*ofs.y;
    if ( l >= (long)sl*sl ) {
      ball->pos.x = stick.x + (((long)sintbl[nd&255]*sl)>>8);
      ball->pos.y = stick.y - (((long)costbl[nd&255]*sl)>>8);
    }
  }
  ofd = nd - ball->pd;
  if ( ofd > 128 ) ofd -= 256;
  else if ( ofd < -128 ) ofd += 256;
  if ( ofd<64 && ofd>-64 ) ball->d += ofd;
  ball->strLgt = (STR_LGT<<RSL)-(abs(ball->d)<<1);
  ball->pd = nd;

  if ( ball->strLgt < (BALL_RADIUS<<RSL<<1) ) {
    ball->strLgt = (BALL_RADIUS<<RSL<<1);
    windCnt++;
    stkIdx = 3;
  } else {
    int stk = 3-(ball->strLgt/(STR_LGT<<RSL>>2));
    if ( stk > stkIdx ) stkIdx = stk;
  }
}

static void hitCheck(Ball *ball1, Ball *ball2) {
  long l;
  Vector ft;
  Vector ofs;
  Vector *vel;
  ofs.x = ball1->pos.x - ball2->pos.x;
  ofs.y = ball1->pos.y - ball2->pos.y;
  l = (long)ofs.x*ofs.x+(long)ofs.y*ofs.y;
  if ( l <= (long)(BALL_RADIUS<<RSL<<1)*(BALL_RADIUS<<RSL<<1) ) {
    long cs;
    Vector csft;
    vel = &(ball1->vel);
    csft = vctGetElement(vel, &ofs);
    vctSub(vel, &csft);
    vctAdd(&(ball2->vel), &csft);
    vel = &(ball2->vel);
    csft = vctGetElement(vel, &ofs);
    vctSub(vel, &csft);
    vctAdd(&(ball1->vel), &csft);
  }
}

static void drawBall(Ball *ball) {
  int bx, by;
  if ( gameStatus.status == GAME_IN_PROGRESS ) {
    if ( ball->tns ) {
      drawThickLine(ball->pos.x>>RSL, ball->pos.y>>RSL, stick.x>>RSL, stick.y>>RSL);
    } else {
      int cx = (ball->pos.x+stick.x)>>1;
      int cy = (ball->pos.y+stick.y+ball->strLgt-abs(ball->pos.x-stick.x))>>1;
      drawThickLine(ball->pos.x>>RSL, ball->pos.y>>RSL, cx>>RSL, cy>>RSL);
      drawThickLine(cx>>RSL, cy>>RSL, stick.x>>RSL, stick.y>>RSL);
    }
  }
  bx = (ball->pos.x>>RSL)-BALL_RADIUS;
  by = (ball->pos.y>>RSL)-BALL_RADIUS;
  if ( bx>-(BALL_RADIUS<<1) && bx<160 && by>-(BALL_RADIUS<<1) && by<160 ) {
    WinDrawBitmap(objectBitmapPtr[BALL_BITMAP_IDX], bx, by);
  }
}

static void movePen() {
  Int16 px, py;
  Boolean pd;
  int i, j;

  ps.x = stick.x; ps.y = stick.y;
  EvtGetPen(&px, &py, &pd);
  if ( pd ) {
    stick.x = px<<RSL; stick.y = py<<RSL;
  }

  vctSub(&ps, &stick);
  windCnt = 0;
  stkIdx = 0;
  for ( i=0 ; i<ballNum ; i++ ) {
    moveBall(&ball[i]);
  }
  for ( i=0 ; i<ballNum ; i++ ) {
    for ( j=i+1 ; j<ballNum ; j++ ) {
      hitCheck(&ball[i], &ball[j]);
    }
    drawBall(&ball[i]);
  }
  if ( windCnt >= ballNum ) {
    gameStatus.score++;
    okBoard.cnt = OK_CNT;
    okBoard.x = stick.x>>RSL; okBoard.y = stick.y>>RSL;
    if ( (gameStatus.score%3) == 0 ) {
      ballNum++;
      if ( ballNum > BALL_MAX ) ballNum = BALL_MAX;
    }
    initAllBalls();
    if ( gameStatus.score >= 10 ) {
      gameStatus.status = GAME_END;
    }
  }
  
  if ( gameStatus.status == GAME_END ) stkIdx = 0;
  WinDrawBitmap
    (objectBitmapPtr[STICK1_BITMAP_IDX+stkIdx], 
     (stick.x>>RSL)-stkIdx-1, (stick.y>>RSL)-stkIdx-1);
}

void handleGameStatus() {
  if ( gameStatus.status != TITLE ) {
    startDrawOffscreen();
    clearDisplay();
    movePen();
    if ( okBoard.cnt >= 0 ) moveOkBoard();
    drawShou(gameStatus.score);
    endDrawOffscreen();
    flipDisplay();
    if ( gameStatus.status == GAME_IN_PROGRESS ) {
      gameStatus.time++;
    } else {
      gameStatus.endCnt++;
      if ( gameStatus.endCnt > 12 ) initTitle();
    }
  }
}
