/**
 * Machikomake
 * (c) Kenta Cho (cs8k-cyu@asahi-net.or.jp)
 * http://www.asahi-net.or.jp/~cs8k-cyu/
 *
 * mckm.c(main routine)
 */

#include <PalmOS.h>
#include "mckm.h"

/**
 * event hanlder for game form
 */
static Boolean GameFormHandleEvent(EventPtr event) {
  Boolean handled = FALSE;
  switch ( event->eType ) {
  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    startGame();
    handled = TRUE;
    break;
  }
  return handled;
}

/**
 * event hanlder for title form
 */
static Boolean TitleFormHandleEvent(EventPtr event) {
  Boolean handled = FALSE;
  switch ( event->eType ) {
  case ctlSelectEvent:
    if ( event->data.ctlSelect.controlID == START_BUTTON ) {
      initGame();
      handled = TRUE;
    }
    break;
  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    startTitle();
    handled = TRUE;
    break;
  }
  return handled;
}

/**
 * activate forms
 */
static Boolean ApplicationHandleEvent(EventPtr event) {
  FormPtr frm;
  Int16 formID;
  Boolean handled = FALSE;

  if ( event->eType == frmLoadEvent ) {
    formID = event->data.frmLoad.formID;
    frm = FrmInitForm(formID);
    FrmSetActiveForm(frm);
    switch ( formID ) {
    case GAME_FORM:
      FrmSetEventHandler(frm, GameFormHandleEvent);
      break;
    case TITLE_FORM:
      FrmSetEventHandler(frm, TitleFormHandleEvent);
      break;
    }
    handled = TRUE;
  }
  return handled;
}

static void EventLoop(void) {
  EventType event;
  UInt16 error;
  long ntc, tc;
  int itv;
  ntc = TimGetTicks() + INTERVAL;
  do {
    itv = ntc - TimGetTicks();
    if ( itv < 1 ) itv = 1;
    EvtGetEvent(&event, itv);
    if ( !SysHandleEvent(&event) ) {
      if ( !ApplicationHandleEvent(&event) ) {
	FrmDispatchEvent(&event);
      }
    }
    tc = TimGetTicks();
    if ( tc >= ntc ) {
      handleGameStatus();
      ntc = ((tc+ntc)>>1) + INTERVAL;
    }
  } while ( event.eType != appStopEvent );
}

static int StartApplication(void) {
  randomize();
  createScreenBuffer();
  loadBitmaps();
  initFirst();
  initTitle();
}

static void StopApplication(void) {
  unloadBitmaps();
}

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags) {
  if ( cmd != sysAppLaunchCmdNormalLaunch ) return 0;
  StartApplication();
  EventLoop();
  StopApplication();
  return 0;
}
